@@ -1,5 +1,5 @@
 /* Copyright (c) 2000  Kevin Sullivan <nite@gis.net>
- *
+1 *
  * Please refer to the COPYRIGHT file for more information.
  */
 
@@ -10,20 +10,414 @@
 #include <ncurses.h>
 #include <sys/time.h>
 
-
+#include "colors.h"
 #include "defines.h"
-#include "cmds.h"
+#include "scmds.h"
+#include "scheck.h"
 #include "vfuncs.h"
 #include "alias.h"
+#include "lists.h"
+#include "nap.h"
+#include "sscr.h"
+#include "winio.h"
 
+/* the procedures in this file operate on two data structures: a
+ * linked list of aliases and a linked list of "sets". Here a "set" is
+ * something that is set by the command "set", i.e., a variable.
+ *
+ * alias_t provides for a simple key-value structure, where the key is
+ * a string called "nm" and the value is a string called "args" (the
+ * replacement for alias "nm").
+ *
+ * sets_t provides a similar key-value structure: the key is again a
+ * string called "nm", and the value is a pair of a string "d" and an
+ * integer "t". Apparently the intention is that t==0 if d represents
+ * a number, t==1 otherwise. Actually, this is implemented
+ * inconsistently:
+ * chset() sets t=1 iff either d contains a letter [a-zA-Z] or d=="".
+ * loadsets() sets t=1 iff d contains a non-digit. 
+ *
+ * There is also the array vars, which is defined in vfuncs.c, and
+ * maps certain names to functions.
+ * */
 
 extern vars_t vars[];
 extern WINDOW *wchan; 
+extern info_t info;
 
 alias_t *alhead;
 sets_t *setl;
 
+/* ---------------------------------------------------------------------- */
+/* the first part of this file deals with "sets", or variables that
+   can be "/set" */
 
+hardwired_t hw[] = {
+  { "user", 1, &info.user, "# your napster username" },
+  { "pass", 2, &info.pass, "# your napster password - optional" },
+  { "email", 1, &info.email, "# your napster email address" },
+  { "upload", 1, &info.up, "# list of upload directories, separated by semicolon" },
+  { "download", 1, &info.down, "# your download directory" },
+  { "incomplete", 1, &info.incomplete, "# your directory for incomplete files" },
+  { "maxuploads", 1, &info.maxuploads, "# maximal number of simultaneous uploads allowed" },
+  { "connection", 1, &info.conn,
+    "# your connection speed, according to the following chart:\n"
+    "#\n"
+    "# Connection | Number\n"
+    "# -------------------\n"
+    "# Unknown    |  0\n"
+    "# 14.4       |  1\n"
+    "# 28.8       |  2\n"
+    "# 33.6       |  3\n"
+    "# 56.7       |  4\n"
+    "# 64K ISDN   |  5\n"
+    "# 128K ISDN  |  6\n"
+    "# Cable      |  7\n"
+    "# DSL        |  8\n"
+    "# T1         |  9\n"
+    "# T3 or >    | 10" },
+  { "servers", 1, &info.serverlist, "# list of servers, separated by semicolon" },
+  { "dataport", 1, &info.dataport, "# port or range of ports to use for client-client connections" },
+  { "logfile", 1, &info.logfile, "# log file for transfer logs" },
+  { "logallfile", 1, &info.logallfile, "# log file for logging everything" },
+  { "announcepongs", 0, 0, "# should receipt of PONG packets be announced?" },
+  { "autopurge", 0, 0, "# automatically delete items from up- and download lists?" },
+  { "autoreply", 0, 0, "# a string to send as an automatic reply to incoming user messages" },
+  { "connecttimeout", 0, 0, "# timeout after this many seconds while connecting to server" },
+  { "cursorfollowsscreen", 0, 0, "# should PgUp and PgDn move the screen, rather than the cursor, on the\n# search result screen?" },
+  { "debug", 0, 0, "# debug level" },
+  { "incompletesuffix", 0, 0, "# suffix to use for incomplete files" },
+  { "maxdownloads", 0, 0, "# maximum number of simultaneous downloads allowed" },
+  { "maxdownuser", 0, 0, "# maximum number of simultaneous downloads per user" },
+  { "maxupuser", 0, 0, "# maximum number of simultaneous uploads per user" },
+  { "metaserver", 0, 0, "# URL of a napigator-style metaserver" },
+  { "metatimeout", 0, 0, "# timeout for connecting to metaserver" },
+  { "newstimeout", 0, 0, "# timeout for retrieving news" },
+  { "noechosets", 0, 0, "# should the /set command be quiet?" },
+  { "nometa", 0, 0, "# skip connecting to metaserver on startup?" },
+  { "nonews", 0, 0, "# do you want to skip news about new releases on startup?" },
+  { "noresultscreen", 0, 0, "# should search results be displayed on the main screen?" },
+  { "noscroll", 0, 0, "# should main screen not scroll to bottom automatically on output?" },
+  { "savechannels", 0, 0, "# should your open channels to be saved between sessions?" },
+  { "savepass", 0, 0, "# should password be saved in config file?" },
+  { "sdefaults", 0, 0, "# default keystrokes for search result screen appearance" },
+  { "showtoomanyuploads", 0, 0, "# display a message when upload limit is reached?" },
+  { "turdsize", 0, 0, "# size of the largest incomplete file which will be auto-removed" }
+};
+
+int hwsize = sizeof(hw)/sizeof(hardwired_t);
+
+/* Append sets from file. If overwrite!=0, overwrite any old
+ * values. If overwrite==0, only write values that were previously
+ * unset (this is used for reading the initial config file, so as to
+ * not overwrite anything that was set explicitly on the command
+ * line). Return -1 on i/o error (with errno set), 0 on success, and 1
+ * if there were ill-formed lines. Log progress info to win if verbose
+ * is set. In any case, log warnings to win. */
+
+int loadsets(const char *fn, WINDOW *win, int overwrite, int verbose)
+{
+  FILE *f;
+  char buf[4096], *nm, *val;
+  int len;
+  int warn = 0;
+
+  f = fopen(fn, "r");
+  if (!f)
+    return(-1);
+  
+  while (1)
+  {
+    if (fgets(buf, sizeof(buf), f) == NULL)
+      break;
+    nm = buf;
+    nm = strip(nm);                        /* strip whitespace */
+    if (*nm == '#' || *nm == 0)            /* if comment or empty, skip line */
+      continue;
+    val = strchr(nm, '=');
+    if (!val)
+      goto badline;
+    *val=0;
+    val = strip(val+1);
+    nm = strip(nm);
+
+    /* at this point, nm is the name and val is the value */
+    if (overwrite || !getval(nm)) {
+      chset(nm,val);
+      if (verbose) {
+	wp(win, "%s%s=%s%s\n", BRIGHT(BLUE), nm, val, WHITE);
+	drw(win);
+      }
+    }
+    continue;
+    
+    /* we go here is we found an ill-formed line */
+  badline: 
+    warn = 1;  /* remember that there was a warning */
+    wp(win, "Warning: ill-formed line in config file (ignoring):\n");
+    len = strlen(buf);
+    if (len>0 && buf[len-1]=='\n')
+      buf[len-1]=0;
+    wp(win, "%s\n", buf);
+    drw(win);
+    continue;
+
+  } /* while (1) */
+
+  fclose(f);
+  
+  return warn ? 1 : 0;
+}
+
+/* save sets to file (in format readable by loadsets, with
+   human-readable comments). Return 0 on success, -1 on i/o failure
+   (with errno set). */
+int savesets(const char *fn)
+{
+  FILE *f;
+  sets_t *cur;
+  int i;
+  char *v;
+
+  f = fopen(fn, "w");
+  if (!f)
+    return(-1);
+  
+  fprintf(f, "# nap config file. Note: for boolean options, 1=true, 0=false\n\n");
+
+  /* first deal with hardwired variables */
+  for (i=0; i<hwsize; i++) {
+    if (hw[i].comment)
+      fprintf(f, "%s\n", hw[i].comment);
+    v = getrealval(hw[i].nm);
+    if (v) {
+      if (hw[i].type==2 && nvar("savepass") != 1) {
+	fprintf(f, "%s=%s\n", hw[i].nm, "?");
+      } else {
+	fprintf(f, "%s=%s\n", hw[i].nm, v);
+      }
+    } else {
+      fprintf(f, "#%s=\n", hw[i].nm);
+    }
+    if (hw[i].comment)
+      fprintf(f, "\n");
+  }
+
+  fprintf(f, "# other user variables:\n");
+
+  /* now print all the remaining variables */
+  for (cur=setl;cur;cur=cur->next) {
+    for (i=0; i<hwsize; i++) {
+      if (!strcasecmp(hw[i].nm, cur->nm))
+	goto next;
+    }
+    fprintf(f, "%s=%s\n", cur->nm, cur->d);
+  next:
+  }
+
+  fclose(f);
+  
+  return(0);
+}
+
+/* print a list of current variable assignments */
+void printsets(WINDOW *win) {
+  sets_t *elt;
+  int count = 0;
+  int i;
+  char *v;
+
+  /* first print hardwired sets */
+  for (i=0; i<hwsize; i++) {
+    v = getval(hw[i].nm);
+    if (v) {
+      wp(win, ""BRIGHT(BLUE)"%s = "WHITE"%s\n", hw[i].nm, v);
+      count++;
+    }
+  }
+  
+  /* then print remaining sets */
+  list_forall(elt, setl) {
+    for (i=0; i<hwsize; i++) {
+      if (!strcasecmp(hw[i].nm, elt->nm))
+        goto next;
+    }
+    wp(win, ""BRIGHT(BLUE)"%s = "WHITE"%s\n", elt->nm, elt->d);
+    count++;
+  next:
+  }
+
+  if (!count)
+    wp(win, "No variables are set\n");
+
+  drw(win);
+} 
+
+/* look up set by name, or NULL if not found. Internal use only.  */
+sets_t *findset(const char *nm)
+{
+  sets_t *cur;
+  
+  for (cur=setl; cur && strcasecmp(cur->nm, nm); cur=cur->next);
+  
+  return(cur);
+}
+
+/* add a set to the list. Note: we do not check for duplicates, thus
+   this is for internal use only. External functions should call
+   chset. */
+void addset(const char *nm, const char *val)
+{
+  sets_t *elt;
+  char *endptr;
+  int i;
+
+  /* check if this is a hardwired set */
+  for (i=0; i<hwsize; i++) {
+    if (hw[i].type && !strcasecmp(nm, hw[i].nm)) {
+      free(*hw[i].strp);
+      *hw[i].strp = strdup(val);
+      return;
+    }
+  }
+
+  /* make new assignment */
+  elt = (sets_t *)malloc(sizeof(sets_t));
+  elt->nm = strdup(nm);
+  elt->d = strdup(val);
+  elt->intval = strtol(elt->d, &endptr, 10);
+  if (endptr && *endptr=='\0') { /* string was valid number */
+    elt->t = 0;
+  } else {
+    elt->intval = 0;
+    elt->t = 1;
+  }
+
+  /* now append it to the list */
+  list_append(sets_t, setl, elt);
+}
+
+/* add a set to the list, but remove duplicate first. Set t=1 if this
+   might be a number. If d is NULL, just delete the set. */
+void chset(const char *nm, const char *d)
+{
+  delset(nm);
+  if (d)
+    addset(nm, d);
+}
+
+/* get the string value of a set, or "?" for a hidden value */
+char *getval(const char *nm) {
+  return getval_t_r(nm, NULL, 0);
+}
+
+/* get the string value of a set, even for a hidden value */
+char *getrealval(const char *nm) {
+  return getval_t_r(nm, NULL, 1);
+}
+
+/* get both string and integer values of a named set. Set *t to be the
+   integer value (unless t=NULL). Let d=NULL if not found. */
+char *getval_t(const char *nm, int *t) {
+  return getval_t_r(nm, t, 0);
+}
+
+/* get both string and integer values of a named set. Set *t to be the
+   integer value (unless t=NULL). Let d=NULL if not found. If real==0
+   get "?" for a hidden value, else the actual value. */
+char *getval_t_r(const char *nm, int *t, int real)
+{
+  sets_t *cur;
+  int i;
+  
+  if (t)
+    *t = 1;
+
+  /* check for hardwired sets first */
+  for (i=0; i<hwsize; i++) {
+    if (!strcasecmp(nm, hw[i].nm) && hw[i].type != 0) {
+      if (*hw[i].strp) {
+	if (real==1 || hw[i].type == 1)
+	  return *hw[i].strp;
+	else
+	  return "?";
+      } else 
+	return NULL;
+    }
+  }
+  
+  cur = findset(nm);
+  if (!cur)
+    return NULL;
+  
+  if (t)
+    *t = cur->t;
+  
+  return(cur->d);
+}
+
+/* remove one named set from list */
+void delset(const char *nm)
+{
+  sets_t *cur, *cur1=NULL;
+  int i;
+
+  /* check for hardwired sets first */
+  for (i=0; i<hwsize; i++) {
+    if (!strcasecmp(nm, hw[i].nm) && hw[i].type != 0) {
+      free(*hw[i].strp);
+      *hw[i].strp = NULL;
+      return;
+    }
+  }
+
+  for (cur=setl;cur&&strcasecmp(cur->nm, nm);cur=cur->next)
+    cur1 = cur;
+  
+  if (!cur)
+    return;
+  
+  if (cur1)
+    cur1->next = cur->next;
+  else if (cur->next)
+    setl = cur->next;
+  else
+    setl = NULL;
+  
+  free(cur->d);
+  free(cur->nm);
+  free(cur);
+}
+
+/* lookup set's value as a number, or -1 if we decided it isn't 
+   a number, or if it isn't in the list */
+long nvar(const char *nm)
+{
+  sets_t *cur;
+  int i;
+
+  /* check for hardwired sets first */
+  for (i=0; i<hwsize; i++) {
+    if (!strcasecmp(nm, hw[i].nm) && hw[i].type != 0) {
+      return *hw[i].strp ? strtol(*hw[i].strp, NULL, 10) : -1;
+    }
+  }
+  
+  cur = findset(nm);
+  if (!cur)
+    return(-1);
+  
+  if (cur->t)
+    return(0);
+  
+  return(cur->intval);
+}
+
+/* ---------------------------------------------------------------------- */
+
+/* return a copy of m with characters bp..ep-1 replaced by n. n may be
+   longer or shorter than ep-bp; in fact even ep<bp is permitted.  The
+   only requirement is that ep,bp <= strlen(m). */
 char *ins(char *m, char *n, int bp, int ep)
 {
   char *st;
@@ -37,6 +431,8 @@ char *ins(char *m, char *n, int bp, int ep)
   return(st);
 }
 
+/* tok is a NULL-terminated list of tokens. Print them to wchan (the
+   main window), enclosed in |..|. Currently unused. */
 void prt(char **tok)
 {
   int i;
@@ -58,12 +454,12 @@ void prt(char **tok)
 
 unsigned char *dovars(unsigned char *nm)
 {
-  unsigned char *r=NULL, tp, *t;
+  unsigned char *r=NULL, *t;
   char **vtok=NULL, b[512], **tok;
   int i, j, vcnt=0, cnt, x, y;
   
   if (findset(nm))
-    r = strdup(getval(nm, &tp));
+    r = strdup(getval(nm));
   
   if (strchr(nm, '('))
   {
@@ -185,6 +581,16 @@ unsigned char *dovars(unsigned char *nm)
     return(r);
 }
 
+/* tokenize a string t of the form " x1, x2, ..., xn" into the tokens
+   x1 through xn. Here x1...xn are well-balanced strings (with respect
+   to '(' and ')'). Notes: x1...xn may contain additional commas
+   inside parentheses.  Whitespace is skipped at the beginning and after
+   commas, but not before commas. The behavior is undefined (messy) if
+   x1...xn are not well-balanced. Some tokens may be the empty string;
+   this also happens if t consist of one or more spaces only.
+   Returns: NULL-terminated array of tokens and number of tokens in
+   n. */
+
 char **form_vtoks(char *t, int *n)
 {
   int i, j, k, c=0;
@@ -198,13 +604,13 @@ char **form_vtoks(char *t, int *n)
   
   ret = (char **)malloc(4096);
   
-  for (i=0,j=0;;i++)
+  for (i=0,j=0;;i++)  /* i=token number, j=character number in t. */
   {
     ret[i] = (char *)malloc(1024);
     memset(ret[i], 0, 1024);
-    while (t[j] && t[j] == ' ')
+    while (t[j] && isspace(t[j]))
       j++;
-    for (k=0,c=0;t[j];k++,j++)
+    for (k=0,c=0;t[j];k++,j++) /* k=char. number in ret[i], c=nesting level */
     {
       if (t[j] == ',' && !c)
         break;
@@ -424,6 +830,7 @@ unsigned char *doalias(alias_t *al, char **tok, int cnt)
   return((unsigned char *)ret);
 }
 
+/* save alias list to named file */
 int savealiases(char *fn)
 {
   FILE *f;
@@ -441,41 +848,43 @@ int savealiases(char *fn)
   return(1);
 }
 
+/* load aliases from named file and append to alias list. The format
+   is ad hoc. */
 int loadaliases(char *fn)
 {
   FILE *f;
-  alias_t *cur, *cur1=NULL;
+  alias_t *cur;
   char buf[4096], tb[4096];
   int i, c, j;
+  char *p;
   
   f = fopen(fn, "r");
   if (!f)
     return(-1);
   
-  for (cur=alhead;cur;cur=cur->next)
-    cur1 = cur;
-  
-  while (!feof(f))
+  while (1)
   {
-    memset(buf, 0, sizeof(buf));
-    fgets(buf, sizeof(buf), f);
-    if (*buf == '#')
+    if (fgets(buf, sizeof(buf), f) == NULL)
+      break;
+    if (*buf == '#')   /* skip comments in column 0 */
       continue;
     if (buf[strlen(buf)-1] == '\n')
       buf[strlen(buf)-1] = 0;
-    for (i=0,c=0;buf[i];i++)
+    for (i=0,c=0;buf[i];i++)  /* find the first space, if any */
     {
-      if (buf[i] == ' ')
+      if (isspace(buf[i]))
       {
         c = 1;
         break;
       }
     }
-    
+    /* no space found; continue on next line. Note: no comment allowed
+       here.  */
+
     if (!c)
     {
       buf[strlen(buf)] = ' ';
-      fgets(buf+strlen(buf), sizeof(buf), f);
+      fgets(buf+strlen(buf), sizeof(buf)-strlen(buf), f);
       if (buf[strlen(buf)-1] == '\n')
         buf[strlen(buf)-1] = 0;
     }
@@ -494,9 +903,10 @@ int loadaliases(char *fn)
         while (1)
         {
           memset(tb, 0, sizeof(tb));
-          fgets(tb, sizeof(tb), f);
+          if (fgets(tb, sizeof(tb), f) == NULL)
+	    break;
           for (j=0;tb[j]==' ';j++);
-          if (tb[j] != '#' || feof(f))
+          if (tb[j] != '#')
           {
             strcpy(buf+i+1, tb);
             buf[i] = ' ';
@@ -533,189 +943,38 @@ int loadaliases(char *fn)
       buf[i] = 0;
       i--;
     }
-    cur = (alias_t *)malloc(sizeof(alias_t));
-    for (i=0;buf[i]!=' ';i++);
-    cur->args = strdup(buf+i+1);
-    cur->nm = (char *)malloc(i);
-    cur->next = NULL;
-    if (!cur1)
-      alhead = cur;
-    else
-      cur1->next = cur;
-    cur1 = cur;
-    strncpy(cur->nm, buf, i);
-    cur->nm[i] = 0;
-  }
-  
-  fclose(f);
-  
-  return(1);
-}
-
-int loadsets(char *fn)
-{
-  FILE *f;
-  char buf[4096], *nm;
-  int i, j;
-  unsigned char t;
-  
-  f = fopen(fn, "r");
-  if (!f)
-    return(-1);
-  
-  while (!feof(f))
-  {
-    memset(buf, 0, sizeof(buf));
-    fgets(buf, sizeof(buf), f);
-    if (!strchr(buf, ' '))
+    /* calculate name and value */
+    p = strchr(buf, ' ');
+    if (!p) {
       continue;
-    if (buf[strlen(buf)-1] == '\n')
-      buf[strlen(buf)-1] = 0;
-    for (i=0;buf[i]!=' ';i++);
-    nm = (unsigned char *)malloc(i+1);
-    strncpy(nm, buf, i);
-    nm[i] = 0;
-    i++;
-    for (t=0,j=i;buf[i];i++)
-    {
-      if (!isdigit(buf[i]))
-      {
-        t = 1;
-        break;
-      }
     }
-    addset(nm, buf+j, t);
-    free(nm);
-  }
+    *p = 0;
+    p++;
+
+    /* delete any old alias of the same name */
+    list_unlink_cond(alias_t, alhead, cur, !strcasecmp(cur->nm, buf));
+    if (cur) {
+      free(cur->nm);
+      free(cur->args);
+      free(cur);
+    }
+    /* now create new element of the alias list */
+    cur = (alias_t *)malloc(sizeof(alias_t));
+    cur->nm = strdup(buf);
+    cur->args = strdup(p);
+
+    /* and append it */
+    list_append(alias_t, alhead, cur);
+
+  } /* while (1) */
   
   fclose(f);
   
   return(1);
 }
 
-int savesets(char *fn)
-{
-  FILE *f;
-  sets_t *cur;
-  
-  f = fopen(fn, "w");
-  if (!f)
-    return(-1);
-  
-  for (cur=setl;cur;cur=cur->next)
-    fprintf(f, "%s %s\n", cur->nm, cur->d);
-  
-  fclose(f);
-  
-  return(1);
-}
-
-void addset(unsigned char *nm, unsigned char *val, unsigned char t)
-{
-  sets_t *cur, *cur1=NULL;
-  
-  if (!setl)
-  {
-    setl = (sets_t *)malloc(sizeof(sets_t));
-    cur = setl;
-  }
-  else
-  {
-    for (cur=setl;cur;cur=cur->next)
-      cur1 = cur;
-    cur = (sets_t *)malloc(sizeof(sets_t));
-    cur1->next = cur;
-  }
-  
-  cur->nm = strdup(nm);
-  cur->d = strdup(val);
-  cur->t = t;
-  cur->next = NULL;
-}
-
-void chset(unsigned char *nm, unsigned char *d)
-{
-  unsigned char t=0;
-  int i;
-  
-  delset(nm);
-  
-  for (i=0;d[i];i++)
-  {
-    if (isalpha(d[i]))
-    {
-      t = 1;
-      break;
-    }
-  }
-  
-  if (!strlen(d))
-    t = 1;
-  
-  addset(nm, d, t);
-}
-
-sets_t *findset(unsigned char *nm)
-{
-  sets_t *cur;
-  
-  for (cur=setl;cur&&strcasecmp(cur->nm, nm);cur=cur->next);
-  
-  return(cur);
-}
-
-unsigned char *getval(unsigned char *nm, unsigned char *t)
-{
-  sets_t *cur;
-  
-  cur = findset(nm);
-  if (!cur)
-  {
-    *t = 0;
-    return(NULL);
-  }
-  
-  *t = cur->t;
-  
-  return(cur->d);
-}
-
-void delset(unsigned char *nm)
-{
-  sets_t *cur, *cur1=NULL;
-  
-  for (cur=setl;cur&&strcasecmp(cur->nm, nm);cur=cur->next)
-    cur1 = cur;
-  
-  if (!cur)
-    return;
-  
-  if (cur1)
-    cur1->next = cur->next;
-  else if (cur->next)
-    setl = cur->next;
-  else
-    setl = NULL;
-  
-  free(cur->d);
-  free(cur->nm);
-  free(cur);
-}
-
-long nvar(unsigned char *nm)
-{
-  sets_t *cur;
-  
-  cur = findset(nm);
-  if (!cur)
-    return(-1);
-  
-  if (cur->t)
-    return(-1);
-  
-  return(atol(cur->d));
-}
-
+/* evaluate a (presumably boolean) expression. Used by such things as
+   "/if" and "/while" */
 char *strev(char *str, unsigned char l, unsigned char r, char *(*func)())
 {
   char *s1, *s2, *rt, *t, *t2;
