@@ -17,22 +17,23 @@
 #include "colors.h"
 #include "codes.h"
 #include "scheck.h"
-#include "cmds.h"
+#include "scmds.h"
 #include "nap.h"
+#include "winio.h"
 #include "irc.h"
-
+#include "alias.h"
 
 extern info_t info;
 extern chans_t *chanl, *curchan, *recent;
-extern unsigned char wmode, finp;
-extern int curx, tind;
-extern WINDOW *winput;
+extern int wmode;
+extern int tind;
+extern int screen;
 
 int ircsock = 0;
 char *nbuf;
 char *mnick = NULL;
-unsigned char ircin=0, ircmode=0, nwho=0;
-char cbuf[4096] = "\0";
+int ircin=0, ircmode=0, nwho=0;
+char ircbuf[4096] = "\0";
 int cpos = 0, cport = -1;
 
 in_irc_cmd_t handlers[] = {
@@ -114,11 +115,11 @@ int dccfunc(WINDOW *win, sock_t *m)
   
   for (i=0;i<1024;i++)
   {
-    if (read(m->s, msg+i, 1) <= 0)
+    if (read(m->fd, msg+i, 1) <= 0)
     {
       wp(win, "%s* DCC Connection to %s closed%s\n", RED, nm, WHITE);
       drw(win);
-      close(m->s);
+      close(m->fd);
       return(0);
     }
     if (msg[i] == '\n')
@@ -143,10 +144,10 @@ int ldcc(WINDOW *win, sock_t *m)
   int frmlen = sizeof(frm);
   char *nm=NULL;
   
-  s = accept(m->s, (struct sockaddr *)&frm, &frmlen);
+  s = accept(m->fd, (struct sockaddr *)&frm, &frmlen);
   if (s == -1)
   {
-    close(m->s);
+    close(m->fd);
     return(0);
   }
   
@@ -158,16 +159,15 @@ int ldcc(WINDOW *win, sock_t *m)
   wp(win, "%s* DCC Connection to %s established%s\n", BRIGHT(CYAN), strchr(m->nm, ' ')+1, WHITE);
   drw(win);
   
-  close(m->s);
+  close(m->fd);
   return(0);
 }
 
 int inirc(WINDOW *win, sock_t *m)
 {
-  int s = m->s, i, r, cnt, pt, j;
+  int s = m->fd, i, r, cnt, pt, j;
   char buf[4096], **tok;
   sock_t *t;
-  cmds_t *cur;
   
   memset(buf, 0, sizeof(buf));
   
@@ -181,34 +181,26 @@ int inirc(WINDOW *win, sock_t *m)
     ircin = 0;
     nwho = 0;
     delircchans();
-    if (wmode && !finp)
+    if (wmode && screen==MAIN_SCREEN)
     {
       dscr(win);
       drw(win);
+      indraw();
+      dstatus();
     }
-    t = findsock("input");
-    if (t)
-    {
-      cur = (cmds_t *)t->d;
-      if (cur->cmd)
-      {
-        indraw(cur->cmd, curx, winput);
-        drw(winput);
-      }
-    }
+
     ircmode = 0;
-    dstatus();
     mnick = NULL;
     return(0);
   }
   
-  if (!strchr(buf, '\n') && !(*cbuf))
+  if (!strchr(buf, '\n') && !(*ircbuf))
   {
-    r = recv(s, cbuf, j, 0);
+    r = recv(s, ircbuf, j, 0);
     cpos += r;
     return(1);
   }
-  else if (!(*cbuf))
+  else if (!(*ircbuf))
   {
     for (i=0;buf[i]!='\n';i++);
     memset(buf, 0, sizeof(buf));
@@ -216,17 +208,17 @@ int inirc(WINDOW *win, sock_t *m)
   }
   else if (!strchr(buf, '\n'))
   {
-    r = recv(s, ((char *)cbuf)+cpos, j, 0);
+    r = recv(s, ((char *)ircbuf)+cpos, j, 0);
     cpos += r;
     return(1);
   }
   else
   {
     for (i=0;buf[i]!='\n';i++);
-    r = recv(s, ((char *)cbuf)+cpos, i+1, 0);
+    r = recv(s, ((char *)ircbuf)+cpos, i+1, 0);
     cpos = 0;
-    strcpy(buf, cbuf);
-    memset(cbuf, 0, sizeof(cbuf));
+    strcpy(buf, ircbuf);
+    memset(ircbuf, 0, sizeof(ircbuf));
   }
   
   if (buf[strlen(buf)-1] == '\n')
@@ -234,7 +226,7 @@ int inirc(WINDOW *win, sock_t *m)
   if (buf[strlen(buf)-1] == '\r')
     buf[strlen(buf)-1] = 0;
   
-  if (info.d == 2)
+  if (nvar("debug") == 2)
   {
     wp(win, "<-- |%s|\n", buf);
     drw(win);
@@ -251,7 +243,7 @@ int inirc(WINDOW *win, sock_t *m)
   {
     if (!handlers[i].text)
     {
-      if (info.d == 1)
+      if (nvar("debug") == 1)
       {
         wp(win, "<-- |%s|\n", buf);
         drw(win);
@@ -270,12 +262,7 @@ int inirc(WINDOW *win, sock_t *m)
       t = findsock("input");
       if (t)
       {
-        cur = (cmds_t *)t->d;
-        if (cur->cmd)
-        {
-          indraw(cur->cmd, curx, winput);
-          drw(winput);
-        }
+	indraw();
       }
       dstatus();
       return(r);
@@ -1196,10 +1183,9 @@ I_IRC_FUNC(hprivmsg)
   else if (!strcasecmp(tok[3]+2, "DCC") && tok[4] && !strcasecmp(tok[4], "CHAT") && tok[5] && tok[6] && tok[7])
   {
     msprintf(&nm, "pending %s", nk);
-    addsock(-1, nm, 0, NULL);
-    sk = findsock(nm);
-    sk->d = NULL;
-    msprintf((char **)&sk->d, "%s %s", tok[6], tok[7]);
+    sk = addsock(-1, nm, 0, NULL);
+    sk->dxx = NULL;
+    msprintf(&sk->dxx, "%s %s", tok[6], tok[7]);
     wp(win, "%s* %s has requested a DCC chat%s\n", BRIGHT(CYAN), nk, WHITE);
     drw(win);
     free(nm);
@@ -1421,7 +1407,7 @@ O_IRC_FUNC(imsg)
       drw(win);
       return;
     }
-    ssock(sk->s, "%s\n", cstr(str, 2));
+    ssock(sk->fd, "%s\n", cstr(str, 2));
     wp(win, "%s* --> [%s(%s)%s]%s %s\n", GREEN, WHITE, tok[1]+1, GREEN, WHITE, cstr(str, 2));
     drw(win);
     return;
@@ -1608,7 +1594,7 @@ O_IRC_FUNC(ipart)
     sk = findsock("server");
     if (sk)
     {
-      sendpack(sk->s, F_PART, pt->nm);
+      sendpack(sk->fd, F_PART, "%s", pt->nm);
       pt->p |= 1;
     }
     return;
@@ -1729,14 +1715,11 @@ O_IRC_FUNC(inap)
   if (!sk)
     ns = -1;
   else
-    ns = sk->s;
+    ns = sk->fd;
   
   ircmode = 0;
   r = parseout(ns, cstr(str, 1), win);
   ircmode = t;
-  
-  if (r == -1)
-    delsock(s);
 }
 
 O_IRC_FUNC(ictcp)
@@ -1840,8 +1823,8 @@ O_IRC_FUNC(idcc)
     nm = NULL;
     if (sk)
     {
-      close(sk->s);
-      delsock(sk->s);
+      close(sk->fd);
+      delsock(sk->fd);
       wp(win, "%s* Closed active DCC chat to %s%s\n", RED, tok[2], WHITE);
       drw(win);
       return;
@@ -1852,9 +1835,9 @@ O_IRC_FUNC(idcc)
     nm = NULL;
     if (sk)
     {
-      close(sk->s);
-      free(sk->d);
-      delsock(sk->s);
+      close(sk->fd);
+      free(sk->dxx);
+      delsock(sk->fd);
       wp(win, "%s* Rejected pending DCC chat to %s%s\n", RED, tok[2], WHITE);
       drw(win);
       ssock(s, "NOTICE %s :\001DCC REJECT CHAT chat\001\n", tok[2]);
@@ -1866,8 +1849,8 @@ O_IRC_FUNC(idcc)
     nm = NULL;
     if (sk)
     {
-      close(sk->s);
-      delsock(sk->s);
+      close(sk->fd);
+      delsock(sk->fd);
       wp(win, "%s* Closed waiting DCC chat to %s%s\n", RED, tok[2], WHITE);
       drw(win);
       return;
@@ -1933,15 +1916,15 @@ O_IRC_FUNC(ichat)
     return;
   }
   
-  sscanf(sk->d, "%lu %i", &addr, &port);
+  sscanf(sk->dxx, "%lu %i", &addr, &port);
   sock = socket(AF_INET, SOCK_STREAM, 0);
   
   dst.sin_addr.s_addr = htonl(addr);
   dst.sin_port = htons(port);
   dst.sin_family = AF_INET;
   
-  free(sk->d);
-  delsock(sk->s);
+  free(sk->dxx);
+  delsock(sk->fd);
   
   if (connect(sock, (struct sockaddr *)&dst, sizeof(dst)) == -1)
   {
